@@ -1,6 +1,7 @@
 from shutil import copy
 from os.path import isdir
 
+from setux.core.errors import ExecError
 from setux.core.target import Target
 from . import error, info
 
@@ -15,8 +16,11 @@ class Local(Target):
 
     def run(self, *arg, **kw):
         arg, kw = self.parse(*arg, **kw)
+        check = kw.pop('check', False)
         ret, out, err =  super().run(*arg, **kw)
         self.trace(' '.join(arg), ret, out, err, **kw)
+        if check and ret:
+            raise ExecError(' '.join(arg), ret, out, err)
         return ret, out, err
 
     def __call__(self, command, **kw):
@@ -25,6 +29,9 @@ class Local(Target):
         print('\n'.join(err))
         return ret
 
+    def chk_cnx(self):
+        return True
+
     def read(self, path, mode='rt', critical=True, report='normal'):
         if report=='normal':
             info(f'\tread {path}')
@@ -53,7 +60,7 @@ class Local(Target):
         assert dst
         self.Dir(dst).set()
         info(f'\tsync {src} -> {dst}')
-        self.rsync(f'{src} {dst}')
+        return self.rsync(f'{src} {dst}')
 
     def export(self, path):
         error("can't export on local")
