@@ -1,9 +1,10 @@
 from tempfile import NamedTemporaryFile
-from os.path import isdir
-import sys
+from os.path import isdir, basename
+from importlib import import_module
 
+from setux.core.errors import ExecError
 from setux.core.target import Target
-from . import logger, info
+from . import logger, info, error
 from . import remote_tpl
 
 
@@ -27,6 +28,7 @@ class SSH(Target):
 
     def run(self, *arg, **kw):
         arg, kw = self.parse(*arg, **kw)
+        check = kw.pop('check', False)
         cmd = ['ssh']
         if self.priv: cmd.extend(['-i', self.priv])
         cmd.append(f'{self.user}@{self.host}')
@@ -35,13 +37,28 @@ class SSH(Target):
         kw['skip'] = self.skip
         ret, out, err =  super().run(*cmd, **kw)
         self.trace(' '.join(arg), ret, out, err, **kw)
+        if check and ret:
+            raise ExecError(' '.join(arg), ret, out, err)
         return ret, out, err
 
     def __call__(self, command, **kw):
         ret, out, err = self.run(f'"{command}"', **kw)
-        print('\n'.join(out))
+        info('\n\t'.join(out))
         return ret
 
+    def chk_cnx(self):
+        ret, out, err = self.run('uname', report='quiet')
+        if ret == 0:
+            return True
+        else:
+            key = f'-i {self.priv} ' if self.priv else ''
+            msg = [
+                f' {self.name} ! connection error !',
+                f'ssh {key}{self.user}@{self.host}\n',
+            ]
+            error('\n'.join(msg))
+            return False
+
     def scp(self, *arg, **kw):
         arg, kw = self.parse(*arg, **kw)
         cmd = ['scp']
@@ -54,19 +71,27 @@ class SSH(Target):
     def send(self, local, remote=None, quiet=False):
         if not remote: remote = local
         if not quiet: info(f'\tsend {local} -> {remote}')
+        dest = remote[:remote.rfind('/')]
+        self.run(f'mkdir -p {dest}', report='quiet')
         self.scp(f'{local} {self.user}@{self.host}:{remote}')
 
     def fetch(self, remote, local, quiet=False):
         if not quiet: info(f'\tfetch {local} <- {remote}')
         self.scp(f'{self.user}@{self.host}:{remote} {local}')
 
+    def rsync_opt(self):
+        if self.priv:
+            return f'-e "ssh -i {self.priv}"'
+        else:
+            return '-e ssh'
+
     def sync(self, src, dst=None):
         assert isdir(src), f'\n ! sync reqires a dir ! {src} !\n'
         if not src.endswith('/'): src+='/'
         if not dst: dst = src
-        self.Dir(dst[:-1]).set()
+        self.distro.dir(dst[:-1]).set()
         info(f'\tsync {src} -> {dst}')
-        self.rsync(f'{src} {self.user}@{self.host}:{dst}')
+        return self.rsync(f'{src} {self.user}@{self.host}:{dst}')
 
     def read(self, path, mode='rt', critical=True, report='normal'):
         if report=='normal':
@@ -87,33 +112,26 @@ class SSH(Target):
             self.send(tmp.name, path, quiet=True)
         return self.read(path, mode=mode.replace('w','r'), report='quiet') == content
 
-    def export(self, path):
-        info(f'\texport -> {path}')
-        todo = set()
-        for  name, module in self.modules.items.items():
-            if name.startswith('setux'): continue
-            base = name.split('.')[0]
-            full = sys.modules[name].__file__
-            for i in range(len(full)):
-                root = full[:i]
-                if root.endswith(base):
-                    todo.add(root)
-                    break
-        exported = list()
-        for root in todo:
-            name = root.split('/')[-1]
-            dest = '/'.join((path, name))
-            self.sync(root, dest)
-            exported.append(name)
-        return exported
+    def export(self, name, root):
+        info(f'\texport {name} -> {root}')
+        cls = self.modules.items[name]
+        mod = cls(self.distro)
+        for module in mod.submodules:
+            self.export(module, root)
+        full = import_module(cls.__module__).__file__
+        name = basename(full)
+        self.send(
+            full,
+            f'{root}/setux/modules/{name}',
+        )
 
     def remote(self, module, export_path=None, **kw):
         with logger.quiet():
-            self.Pip.install('setux')
-            path = export_path or '/tmp/setux'
+            self.pip.install('setux_repl')
+            path = export_path or '/tmp/setux/import'
             name = 'exported.py'
-            exported = self.export(path)
-            kwargs = ', '.join(f"{k}='{v}'" for k,v in kw.items()) if kw else ''
+            self.export(module, path)
+            kwargs = ', '+', '.join(f"{k}='{v}'" for k,v in kw.items()) if kw else ''
             self.write('/'.join((path, name)), remote_tpl.deploy.format(**locals()))
             ret, out, err = self.script(remote_tpl.script.format(**locals()))
         info('\t'+'\n\t'.join(out))
